#!/usr/bin/perl -w
#Copyright 2000 Cobalt Networks
#Author: Philip Martin
#14 Nov 2000
#
#mail module that will auto-detect the current encoding 
#and language, add the correct headers and convert the
#encoding as needed to conform to standards.

#Currently English and Japanese are supported.
#To add a language please see the _encodeBody
#and _encodeHeader functions.


package I18nMail;

sub false(){0}
sub true(){1}

use vars qw/$VERSION/;
use Jcode;  #Jcode kicks ass.

$VERSION="1.0";


$encodingTable = {
	ja => {
		charset => 'ISO-2022-JP',
		schemes => {
			body => [
				\&I18nMail::Subs::toIso2022Jp,
				\&I18nMail::Subs::setBodyLang
				],

			header => {
				"default" => [  #default encoding scheme
					\&I18nMail::Subs::toIso2022Jp,
					\&I18nMail::Subs::Base64
				      ],
				"email" => [ #encoding for headers w/ email addrs
					\&I18nMail::Subs::getEmailName,
					\&I18nMail::Subs::toIso2022Jp,
					\&I18nMail::Subs::Base64,
					\&I18nMail::Subs::glueEmailTogether
					   ]
				  }
			   }

		},
	en => {
		charset => 'US-ASCII',
		schemes => {
			body => [\&I18nMail::Subs::setBodyLang],
			header => {
				"default" => [\&I18nMail::Subs::QuotedPrintable],
				"email" => [
					\&I18nMail::Subs::getEmailName,
					\&I18nMail::Subs::QuotedPrintable,
					\&I18nMail::Subs::glueEmailTogether
					   ]
				  }
			   }
	      }

};

#constructor
#takes 1 optional argument, a two letter
#language code.
#without that argument the module will
#auto-detect the encoding, and therefore
#language, of each peice of data.
#(note that I said each piece of data,
#this module will let you have english
#text in the subject and japanese text
#in the body and will encode each one
#differently (which is correct according
#to the RFCs.))

sub new{
	my($class, $lang)=@_;
	my $self={};
	bless($self, $class);
	$self->{_lang}=$lang if defined $lang;
	$self->{_bodyLang}=$lang if defined $lang;
	$self->{_encodingTable}=$encodingTable;
	return $self;
}

sub setLang{
	my($self,$lang)=@_;
	$lang = substr($lang, 0, 2); # strip off trailing bits
	$self->{_lang}=$lang;
	$self->{_bodyLang}=$lang;
	return true;
}

sub addEncoding{
	my($self,$lang,$table)=@_;
	$self->{_encodingTable}->{$lang}=$table;
	return true;
}

sub setBody{
	my ($self,$body,$contentType)=@_;
	$contentType="text/plain" unless defined $contentType;

	$self->{_contentType}=$contentType;
	$body=~s/\r(?!\n)/\n/g; #workaround little Jcode bug
	$body=~s/(?<!\r)\n/\r\n/g;
	$self->{_body}=$self->_encode($body,"body");

	unless(defined $self->{_body}){
		return false;
	}
	return true;
}

sub setSubject{
	my($self,$subject)=@_;
	chomp($subject);
	$self->{_subject}=$self->_encode($subject,"header","default");

	unless(defined $self->{_subject}){
		return false;
	}
	return true;
}

sub addRawTo{
	my $self=shift;

	foreach(@_){
		$self->{_to} .= $_.", " unless $_ eq "";
	}
	unless(defined $self ->{_to}){
		return false;
	}
	return true;
}

sub addTo{
	my $self=shift;
	
	foreach(@_){
		chomp;
		$self->{_to} .= $self->_encode($_,"header","email").", " unless $_ eq "";
	}
	unless(defined $self->{_to}){
		return false;
	}
	return true;
}

sub addCc{
	my $self=shift;
	
	foreach(@_){
		chomp;
		$self->{_cc} .= $self->_encode($_,"header","email").", " unless $_ eq "";
	}
	unless(defined $self->{_cc}){
		return false;
	}
	return true;
}

sub addBcc{
	my $self=shift;
	
	foreach(@_){
		chomp;
		$self->{_bcc} .= $self->_encode($_,"header","email").", " unless $_ eq "";
	}
	unless(defined $self->{_bcc}){
		return false;
	}
	return true;
}

sub setFrom{
	my($self,$from)=@_;
	chomp $from;
	$self->{_from} .= $self->_encode($from,"header","email");

	unless(defined $self->{_from}){
		return false;
	}

	return true;
}

sub toText{
	my($self)=@_;
	my($msg,$charset);
	$charset=$self->{_encodingTable}->{$self->{_bodyLang}}->{charset}; #  :)

	$msg  = "From: $self->{_from}\r\n";
	$msg .= "To: $self->{_to}\r\n";
	$msg .= "Cc: $self->{_cc}\r\n" if defined $self->{_cc};
	$msg .= "Bcc: $self->{_bcc}\r\n" if defined $self->{_bcc};
	$msg .= "Date: ".scalar(localtime(time()))."\r\n";
	$msg .= "Subject: $self->{_subject}\r\n";
	$msg .= "Content-Language: $self->{_bodyLang}\r\n";
	$msg .= "Content-Type: $self->{_contentType}; charset=\"$charset\"\r\n";
	$msg .= "\r\n";
	$msg .= $self->{_body};
	$msg .= "\r\n";

	return $msg;
}

########################################
#PRIVATE FUNCTIONS

sub _detectLang{
	my($self,$str)=@_;

	my %enc2lang=(
		sjis=>'ja',
		jis=>'ja',
		euc=>'ja',
		ascii=>'en',
	);

	$enc=Jcode::getcode($str);
	if(exists $enc2lang{$enc}){
		return $enc2lang{$enc};
	}else{
		return undef;
	}
}


sub _encode{
	my($self,$str,$type,$subType)=@_;
	my($lang,$code,$charset,$ret);

	$lang=$self->{_lang} || $self->_detectLang($str);
	return undef unless defined $lang;
	return undef unless exists $self->{_encodingTable}->{$lang};


	if(defined $subType){
		$code=$self->{_encodingTable}->{$lang}->{schemes}->{$type}->{$subType};
	}else{
		$code=$self->{_encodingTable}->{$lang}->{schemes}->{$type};
	}
	return undef unless ref $code eq "ARRAY";

	$charset=$self->{_encodingTable}->{$lang}->{charset};
	$ret=$str;

	foreach my $func (@$code){
		$ret=$func->($ret,$charset,$str,$lang,$self);
	}
	return $ret;
}


package I18nMail::Subs;

use Jcode;
use MIME::Base64;
use MIME::QuotedPrint;

sub getEmailName{
	my($email)=@_;
	my($name,$mailaddr);

	$mailaddr=q/[^()<>@,;:\\\\"\\[\\]\\s\\x00-\\x1f\\x7f]+(?:@[a-zA-Z0-9._-]+)?/;

	if($email=~/^$mailaddr$/){
		return $email;
	}elsif($email=~/<.*>/ && $email=~/^([^<]+)<$mailaddr>$/){
		return $1;
	}elsif($email=~/\(.*\)/ && (($name)=($email=~/\((.*)\)/))){
		return $name;
	}else{
		return undef;
	}
}

sub getEmailAddr{
	my($email)=@_;
	my($addr,$mailaddr);

	$mailaddr=q/[^()<>@,;:\\\\"\\[\\]\\s\\x00-\\x1f\\x7f]+(?:@[a-zA-Z0-9._-]+)?/;

	if($email=~/^($mailaddr)$/){
		return $1;
	}elsif($email=~/<($mailaddr)>/){
		return $1;
	}else{
		return undef;
	}
}

sub Base64{
	my($str,$charset)=@_;
	my $enc_str=encode_base64($str);
	chomp($enc_str);
	return "=?$charset?B?".$enc_str."?=";
}

sub QuotedPrintable{
	my($str,$charset)=@_;
	my $qp_str=encode_qp($str);
	chomp($qp_str);
	return "=?$charset?Q?".$qp_str."?=";
}

sub toIso2022Jp{
	my($str)=@_;
	return Jcode->new($str)->iso_2022_jp;
}

sub setBodyLang{
	my($ret,$lang,$self)=@_[0,3,4];
	$self->{_bodyLang}=$lang;
	return $ret;
}

sub glueEmailTogether{
	my($ret,$str)=@_[0,2];
	return $ret." <".getEmailAddr($str).">";
}
1;
